@@ -34,6 +34,7 @@
 #include <linux/version.h>
 #include <linux/i8042.h>
 #include <linux/serio.h>
+#include <acpi/battery.h>
 #include "uniwill_interfaces.h"
 #include "uniwill_leds.h"
 
@@ -131,6 +132,19 @@ int uniwill_read_ec_ram_with_retry(u16 address, u8 *data, int retries)
 }
 EXPORT_SYMBOL(uniwill_read_ec_ram_with_retry);
 
+static int uniwill_read_ec_ram_u16(u16 hibyte_address, u16 lobyte_address, u16 *data) {
+	int result;
+	u8 hi, lo;
+	result = uniwill_read_ec_ram(hibyte_address, &hi);
+	if (result)
+		return result;
+	result = uniwill_read_ec_ram(lobyte_address, &lo);
+	if (result)
+		return result;
+	*data = (hi << 8) | lo;
+	return result;
+}
+
 int uniwill_write_ec_ram(u16 address, u8 data)
 {
 	int status;
@@ -1180,6 +1194,102 @@ static ssize_t uw_usb_powershare_store(struct device *child,
 		return -EIO;
 }
 
+static ssize_t raw_cycle_count_show(struct device *device,
+				struct device_attribute *attr,
+				char *buf)
+{
+	int result;
+	u16 cycle_count;
+	result = uniwill_read_ec_ram_u16(UW_EC_REG_BATTERY_CYCN_HI, UW_EC_REG_BATTERY_CYCN_LO, &cycle_count);
+	if (result)
+		return result;
+	return snprintf(buf, PAGE_SIZE, "%d\n", cycle_count);
+}
+
+static ssize_t raw_xif1_show(struct device *device,
+				struct device_attribute *attr,
+				char *buf)
+{
+	int result;
+	u16 xif1;
+	result = uniwill_read_ec_ram_u16(UW_EC_REG_BATTERY_XIF1_HI, UW_EC_REG_BATTERY_XIF1_LO, &xif1);
+	if (result)
+		return result;
+	return snprintf(buf, PAGE_SIZE, "%d\n", xif1);
+}
+
+static ssize_t raw_xif2_show(struct device *device,
+				struct device_attribute *attr,
+				char *buf)
+{
+	int result;
+	u16 xif2;
+	result = uniwill_read_ec_ram_u16(UW_EC_REG_BATTERY_XIF2_HI, UW_EC_REG_BATTERY_XIF2_LO, &xif2);
+	if (result)
+		return result;
+	return snprintf(buf, PAGE_SIZE, "%d\n", xif2);
+}
+
+static DEVICE_ATTR_RO(raw_cycle_count);
+static DEVICE_ATTR_RO(raw_xif1);
+static DEVICE_ATTR_RO(raw_xif2);
+
+static struct attribute *uw_battery_attrs[] = {
+	&dev_attr_raw_cycle_count.attr,
+	&dev_attr_raw_xif1.attr,
+	&dev_attr_raw_xif2.attr,
+	NULL,
+};
+
+ATTRIBUTE_GROUPS(uw_battery);
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)
+static int uw_battery_add(struct power_supply *battery)
+#else
+static int uw_battery_add(struct power_supply *battery, struct acpi_battery_hook *hook)
+#endif
+{
+	TUXEDO_DEBUG("uw_battery_add\n");
+	if (device_add_groups(&battery->dev, uw_battery_groups))
+		return -ENODEV;
+
+	return 0;
+}
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)
+static int uw_battery_remove(struct power_supply *battery)
+#else
+static int uw_battery_remove(struct power_supply *battery, struct acpi_battery_hook *hook)
+#endif
+{
+	TUXEDO_DEBUG("uw_battery_remove\n");
+	device_remove_groups(&battery->dev, uw_battery_groups);
+	return 0;
+}
+
+static struct acpi_battery_hook uw_battery_hook = {
+	.add_battery = uw_battery_add,
+	.remove_battery = uw_battery_remove,
+	.name = "TUXEDO Battery Extension",
+};
+
+static bool uw_battery_hook_registered = false;
+
+static void uw_battery_init(void)
+{
+	battery_hook_register(&uw_battery_hook);
+	uw_battery_hook_registered = true;
+}
+
+static void uw_battery_uninit(void)
+{
+	if (uw_battery_hook_registered)
+		battery_hook_unregister(&uw_battery_hook);
+	else
+		TUXEDO_ERROR("attempted to unregister battery hook which was not registered\n");
+}
+
+
 static bool uw_mini_led_local_dimming_loaded = false;
 static bool uw_mini_led_local_dimming_last_written_value;
 
@@ -1511,6 +1621,7 @@ struct uniwill_device_features_t *uniwill_get_device_features(void)
 		|| dmi_match(DMI_BOARD_NAME, "XxKK4NAx_XxSP4NAx")
 		|| dmi_match(DMI_BOARD_NAME, "X5KK45xS_X5SP45xS")
 		|| dmi_match(DMI_BOARD_NAME, "X6AR55xU")
+		|| dmi_match(DMI_BOARD_NAME, "X5AR45xS")
 #endif
 	;
 
@@ -1729,6 +1840,7 @@ static int uniwill_keyboard_probe(struct platform_device *dev)
 	uw_ac_auto_boot_init(dev);
 	uw_usb_powershare_init(dev);
 	uw_mini_led_local_dimming_init(dev);
+	uw_battery_init();
 
 	// Ignore return value, it just means there is already a filter active
 	// which is fine, because it is probably just the upstream patch of this
@@ -1754,6 +1866,8 @@ static void uniwill_keyboard_remove(struct platform_device *dev)
 
 	if (uw_charging_profile_loaded)
 		sysfs_remove_group(&dev->dev.kobj, &uw_charging_profile_attr_group);
+	
+	uw_battery_uninit();
 
 	uniwill_leds_remove(dev);
 
